import { env, ReservedSQL, CryptoHasher, sql } from 'bun'
import { MIMEType } from 'util';

export enum MultiMediaType {
  video, image
}

export interface Attachment {
  urls: string[],
  type: MultiMediaType
};

export interface Post {
  post: string,
  post_time: string,
  hash: string,
  attachments: Attachment[],
}
interface SQLPostCast {
  hash: string, post: string, post_time: string, images: number[]
};
const SQLPostCast2Post = (obj: SQLPostCast) => {
  let x: Post =
  {
    post: obj.post,
    hash: obj.hash,
    post_time: obj.post_time,
    attachments: []
  };
  if (obj.images && obj.images.length > 0) {
    x.attachments.push(
      {
        type: MultiMediaType.image, urls: obj.images.map(img => {
          return `/img/${obj.hash}_${img}`
        })
      }
    )
  }
  return x
}


export interface NewPost {
  image: Blob[]
  content: string,
}

export async function insert_post(post: NewPost): Promise<[number, string]> {
  let post_hash = new CryptoHasher("sha256");
  post_hash.update(post.content);
  post_hash.update(Date.now().toString())
  let populated = post_hash.digest("base64");
  let [{ id, hash }] = await sql`INSERT INTO niming.posts (post, hash) VALUES (${post.content}, ${populated}) RETURNING id, hash`
  return [id, hash]

}

export async function init_db() {
  await sql.begin(async sql => {
    await sql`CREATE SCHEMA niming`
    await sql`CREATE TABLE niming.posts (id SERIAL PRIMARY KEY, hash char(44) UNIQUE, post VARCHAR(500), post_time TIMESTAMPTZ DEFAULT now())`
    await sql`CREATE TABLE niming.images (id INTEGER PRIMARY KEY REFERENCES niming.posts (id), fileid integer[] )`
  })
}

export class PostFetcher {
  conn: Promise<ReservedSQL>;

  constructor() {
    this.conn = sql.reserve();
  }

  async init() {
    await this.conn.then(async e => {
      await e`DECLARE post_ptr CURSOR WITH HOLD FOR SELECT niming.posts.*, niming.images.fileid
        AS images FROM niming.posts 
        LEFT JOIN niming.images ON niming.images.hash=niming.posts.hash ORDER BY niming.posts.post_time DESC, niming.posts.hash ASC`
    }).catch(err => {
      console.log(err);
    })
  }

  async postgres_fetch() {
    let x: SQLPostCast[] = []
    if (this.conn) {
      x = await (await this.conn)`FETCH 10 IN post_ptr`;
    }
    return x.map(e => {
      return SQLPostCast2Post(e)
    })
  }
}

